import {parser} from "jexl-parser"
import {LRLanguage, LanguageSupport,delimitedIndent, flatIndent, continuedIndent, indentNodeProp,
        foldNodeProp, foldInside} from "@codemirror/language"

export const jexlLanguage = LRLanguage.define({

  name: "jexl",

  parser: parser.configure({

      props: [

          indentNodeProp.add({

              IfStatement: continuedIndent({ except: /^\s*({|else\b)/ }),

              TryStatement: continuedIndent({ except: /^\s*({|catch\b|finally\b)/ }),

              LabeledStatement: flatIndent,

              Block: delimitedIndent({ closing: "}" }),

              ArrowFunction: cx => cx.baseIndent + cx.unit,

              "TemplateString BlockComment": () => null,

              "Statement Property": continuedIndent({ except: /^{/ }),

          }),

          foldNodeProp.add({

              "Block ArrayExpression": foldInside,

              BlockComment(tree) { return { from: tree.from + 2, to: tree.to - 2 }; }

          })

      ]

  }),

  languageData: {

      closeBrackets: { brackets: ["(", "[", "{", "'", '"', "`"] },

      commentTokens: { line: "//", block: { open: "/*", close: "*/" } },

      indentOnInput: /^\s*(?:case |default:|\{|\}|<\/)$/,

      wordChars: "$"

  }

});

export function jexl() {

  let lang = jexlLanguage

  //let completions = snippets.concat(keywords);

  return new LanguageSupport(lang, [

      // jexlLanguage.data.of({

      //     autocomplete: autocomplete.ifNotIn(dontComplete, autocomplete.completeFromList(completions))

      // }),

      // jexlLanguage.data.of({

      //     autocomplete: localCompletionSource

      // }),

  ]);

}